/* BdfServer - Copyright (c) 2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.xml.dyn;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.namespaces.IconSpace;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.base64.Base64;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.text.RelativePath;
import net.mapeadores.util.xml.ns.NameSpace;
import net.mapeadores.util.xml.svg.SvgWriter;


/**
 *
 * @author Vincent Calame
 */
public class Icon32SvgWriter extends SvgWriter {


    private final BdfServer bdfServer;
    private final RelativePath cartouchePath;

    public Icon32SvgWriter(BdfServer bdfServer) {
        this.bdfServer = bdfServer;
        this.cartouchePath = getCartouchePath(bdfServer);
    }

    public void open() throws IOException {
        addNameSpace(NameSpace.XLINK_NAMESPACE);
        openSvg(32, 32);
        appendDefs();
    }

    public void appendBitmapIcon() throws IOException {
        DocStream bitmapIcon = bdfServer.getResourceDocStream(StorageUtils.ICON32);
        String base64;
        try (InputStream is = bitmapIcon.getInputStream()) {
            byte[] byteArray = IOUtils.toByteArray(is);
            base64 = Base64.encodeBase64String(byteArray);
        }
        startImageOpenTag("data:image/png;base64," + base64, 0, 0, 32, 32);
        if (cartouchePath != null) {
            addAttribute("style", "clip-path: url(#cartoucheClip);");
        }
        closeEmptyTag();
    }

    public void appendBitmapCartouche() throws IOException {
        if (cartouchePath == null) {
            return;
        }
        DocStream cartoucheIcon = bdfServer.getResourceDocStream(cartouchePath);
        if (cartoucheIcon == null) {
            return;
        }
        String base64;
        try (InputStream is = cartoucheIcon.getInputStream()) {
            byte[] byteArray = IOUtils.toByteArray(is);
            base64 = Base64.encodeBase64String(byteArray);
        }
        startImageOpenTag("data:image/png;base64," + base64, 16, 16, 16, 16);
        closeEmptyTag();
    }

    public void close() throws IOException {
        closeSvg();
    }

    private void appendDefs() throws IOException {
        openTag("svg:defs");
        startOpenTag("svg:clipPath");
        addAttribute("id", "cartoucheClip");
        endOpenTag();
        startOpenTag("svg:path");
        addAttribute("d", "M 0 0 h 32 v 15 h -16 v 1 h -1 v 16 h -15 Z");
        closeEmptyTag();
        closeTag("svg:clipPath");
        closeTag("svg:defs");
    }

    private void write() throws IOException {
        open();
        appendBitmapIcon();
        appendBitmapCartouche();
        close();
    }

    private static RelativePath getCartouchePath(BdfServer bdfServer) {
        Attribute cartoucheAttribute = bdfServer.getFichotheque().getFichothequeMetadata().getAttributes().getAttribute(IconSpace.CARTOUCHE_KEY);
        if (cartoucheAttribute != null) {
            try {
                return RelativePath.parse(cartoucheAttribute.getFirstValue());
            } catch (ParseException pe) {

            }
        }
        return null;
    }

    public static String getSvgString(BdfServer bdfServer) {
        Icon32SvgWriter svgWriter = new Icon32SvgWriter(bdfServer);
        StringBuilder buf = new StringBuilder(1024);
        svgWriter.setAppendable(buf);
        svgWriter.setIndentLength(0);
        try {
            svgWriter.appendXMLDeclaration();
            svgWriter.write();
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

}
