/* BdfServer_Commands - Copyright (c) 2011-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.EditSession;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfCommandUtils;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.ui.components.IncludeUiBuilder;
import java.text.ParseException;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.tools.parsers.TypoParser;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.attr.AttributeChange;
import net.mapeadores.util.attr.AttributeParser;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelChange;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.TypoOptions;


/**
 *
 * @author Vincent Calame
 */
public class IncludeChangeCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "IncludeChange";
    public final static String COMMANDKEY = "_ CRP-13";
    public final static String CUSTOM_PARAMNAME = "custom";
    public final static String KEY_PARAMNAME = "key";
    public final static String TITLE_PARAMPREFIX = "title/";
    public final static String ATTRIBUTES_PARAMNAME = "attributes";
    private UiComponents uiComponents;
    private IncludeUi includeUi;
    private Corpus corpus;
    private boolean custom = false;
    private LabelChange labelChange;
    private AttributeChange attributeChange;

    public IncludeChangeCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        boolean done;
        try (EditSession session = startEditSession(Domains.CORPUS, COMMANDNAME)) {
            done = updateIncludeUi(session.getBdfServerEditor(), includeUi);
        }
        putResultObject(CORPUS_OBJ, corpus);
        if (done) {
            setDone("_ done.corpus.includechange", includeUi.getName());
        }
    }

    @Override
    protected void checkParameters() throws ErrorMessageException {
        corpus = getMandatoryCorpus();
        checkSubsetAdmin(corpus);
        String key = getMandatory(KEY_PARAMNAME);
        uiComponents = bdfServer.getUiManager().getMainUiComponents(corpus);
        if (FichothequeUtils.isSpecialIncludeName(key)) {
            includeUi = (IncludeUi) uiComponents.getUiComponent(key);
        } else {
            try {
                ExtendedIncludeKey includeKey = ExtendedIncludeKey.parse(key);
                includeUi = (IncludeUi) uiComponents.getUiComponent(includeKey);

            } catch (ParseException pe) {
                throw BdfErrors.wrongParameterValue(KEY_PARAMNAME, key);
            }
        }
        if (includeUi == null) {
            throw BdfErrors.unknownParameterValue(KEY_PARAMNAME, key);
        }
        custom = requestMap.isTrue(CUSTOM_PARAMNAME);
        if (custom) {
            labelChange = BdfCommandUtils.getLabelChange(requestMap, TITLE_PARAMPREFIX, false);
        }
        String attributes = getMandatory(ATTRIBUTES_PARAMNAME);
        this.attributeChange = AttributeParser.parse(attributes);
    }

    private boolean updateIncludeUi(BdfServerEditor bdfServerEditor, IncludeUi includeUi) {
        IncludeUiBuilder includeUiBuilder = IncludeUiBuilder.derive(includeUi);
        if (!custom) {
            includeUiBuilder.clearCustomLabels();
        } else {
            for (Label label : labelChange.getChangedLabels()) {
                Lang currentLang = label.getLang();
                TypoOptions typoOptions = TypoOptions.getTypoOptions(currentLang.toLocale());
                String labelString = label.getLabelString();
                String checkedLabelString = TypoParser.parseTypo(labelString, typoOptions);
                if (!checkedLabelString.equals(labelString)) {
                    CleanedString cleanedString = CleanedString.newInstance(checkedLabelString);
                    if (cleanedString == null) {
                        includeUiBuilder.removeLabel(currentLang);
                        continue;
                    } else {
                        label = LabelUtils.toLabel(currentLang, cleanedString);
                    }
                }
                includeUiBuilder.putLabel(label);
            }
            for (Lang lang : labelChange.getRemovedLangList()) {
                includeUiBuilder.removeLabel(lang);
            }
        }
        includeUiBuilder.changesAttribute(attributeChange);
        bdfServerEditor.putComponentUi(uiComponents, includeUiBuilder.toIncludeUi());
        return true;
    }

}
