/* global BdfApi */

/**
 * Contient des méthodes statiques pour l'interrogation de l'API du serveur BDF.
 * Les options sont les suivantes :
 * - requestParameters : paramètres supplémentaires passés à la requête
 * - callback: fonction appelée à la fin du traitement
 * 
 * @type Object
 */
BdfApi.Ajax = {};


/**
 * Retourne l'identité du l'utilisateur actuellement connecté
 * 
 * @param {BdfApi.Config} apiConfig Configuration à utiliser
 * @param {Object} options Options de la requête
 * @param {function} options.callback Fonction de rappel
 * @param {Object} options.requestParameters Surcharge des paramètres de la requête
 * @param {Object} options.ignoreLogError Ne pas tenir compte des erreurs
 * @returns {undefined}
 */
BdfApi.Ajax.ping = function (apiConfig, options) {
    var ajaxRequestData = BdfApi.initAjaxRequestData("ping", options);
    $.ajax({
        url: apiConfig.getActionUrl("session"),
        method: "GET",
        dataType: "json",
        data: ajaxRequestData,
        xhrFields: {
           withCredentials: true
        },
        success: function (data, textStatus) {
            BdfApi.handleResponseData(data, options, false);
        }
    });
};


/**
 * Déconnecte l'utilisateur en cours
 * 
 * @param {BdfApi.Config} apiConfig Configuration à utiliser
 * @param {Object} options Options de la requête
 * @param {function} options.callback Fonction de rappel
 * @param {Object} options.requestParameters Surcharge des paramètres de la requête
 * @param {Object} options.ignoreLogError Ne pas tenir compte des erreurs
 * @returns {undefined}
 */
BdfApi.Ajax.exit = function (apiConfig, options) {
    var ajaxRequestData = BdfApi.initAjaxRequestData("ping", options);
    ajaxRequestData["bdf-exit"] = 1;
    $.ajax({
        url: apiConfig.getActionUrl("session"),
        method: "GET",
        dataType: "json",
        data: ajaxRequestData,
        xhrFields: {
           withCredentials: true
        },
        success: function (data, textStatus) {
            BdfApi.handleResponseData(data, options, false);
        }
    });
};


/**
 * Récupère le tableau des fiches correspondantes aux critères de sélection
 * 
 * @param {BdfApi.Config} apiConfig Configuration à utiliser
 * @param {Fql.FicheQuery} ficheQuery 
 * @param {Object} options Options de la requête
 * @param {function} options.callback Fonction de rappel
 * @param {Object} options.requestParameters Surcharge des paramètres de la requête
 * @param {Object} options.ignoreLogError Ne pas tenir compte des erreurs
 * @returns {undefined}
 */
BdfApi.Ajax.loadFicheArray = function (apiConfig, ficheQuery, options) {
    var ajaxRequestData = BdfApi.initAjaxRequestData("fiche", options);
    ajaxRequestData.xml = ficheQuery.toXml(-999);
    $.ajax({
        url: apiConfig.getActionUrl("pioche"),
        method: "GET",
        dataType: "json",
        data: ajaxRequestData,
        xhrFields: {
           withCredentials: true
        },
        success: function (data, textStatus) {
            BdfApi.handleResponseData(data, options, true);
        }
    });
};


/**
 * 
 * @param {BdfApi.Config} apiConfig Configuration à utiliser
 * @param {Fql.MotcleQuery} motcleQuery
 * @param {Object} options Options de la requête
 * @returns {undefined}
 */
BdfApi.Ajax.loadMotcleArray = function (apiConfig, motcleQuery, options) {
    var ajaxRequestData = BdfApi.initAjaxRequestData("motcle", options);
    ajaxRequestData.xml = motcleQuery.toXml(-999);
    $.ajax({
        url: apiConfig.getActionUrl("pioche"),
        method: "GET",
        dataType: "json",
        data: ajaxRequestData,
        xhrFields: {
           withCredentials: true
        },
        success: function (data, textStatus) {
            BdfApi.handleResponseData(data, options, true);
        }
    });
};


/**
 * 
 * @param {BdfApi.Config} apiConfig Configuration à utiliser
 * @param {String} thesaurusName Nom du thésaurus
 * @param {Object} options Options de la requête
 * @param {function} options.callback Fonction de rappel
 * @param {Object} options.requestParameters Surcharge des paramètres de la requête
 * @param {Object} options.ignoreLogError Ne pas tenir compte des erreurs
 * @returns {undefined}
 */
BdfApi.Ajax.loadThesaurus = function (apiConfig, thesaurusName, options) {
    var ajaxRequestData = BdfApi.initAjaxRequestData("thesaurus", options);
    ajaxRequestData.thesaurus = thesaurusName;        
    $.ajax({
        url: apiConfig.getActionUrl("thesaurus"),
        method: "GET",
        dataType: "json",
        data: ajaxRequestData,
        xhrFields: {
           withCredentials: true
        },
        success: function (data, textStatus) {
            BdfApi.handleResponseData(data, options, true);
        }
    });
};


BdfApi.Ajax.loadFiches = function (apiConfig, queries, options) {
    var ajaxRequestData = BdfApi.initAjaxRequestData("fiches", options);
    var xml;
    if (typeof queries === "string") {
        xml = queries;
    } else if (queries instanceof Fql.FicheQuery) {
        xml = queries.toXml(-999)
    } else if (Array.isArray(queries)) {
        xml = "<queries>";
        for(let query of queries) {
            if (typeof query === "string") {
                xml += query;
            } else if (query.toXml) {
                xml  += query.toXml(-999);
            }
        }
        xml += "</queries>";
    }
    ajaxRequestData.xml = xml;
    $.ajax({
        url: apiConfig.getActionUrl("selection"),
        method: "GET",
        dataType: "json",
        data: ajaxRequestData,
        xhrFields: {
           withCredentials: true
        },
        success: function (data, textStatus) {
            BdfApi.handleResponseData(data, options, true);
        }
    });
};
